//import javax.swing.*;
import java.awt.*;
import java.util.Vector;

public class PartitionPanel extends java.awt.Panel {
  public PartitionPanel(int m, int n) {
    localCol = m;
    localRow = n;	    

    setupGUI();
    //updateData();
  }

  public PartitionPanel(int _id, int _nProcCol, int _nProcRow, int _matrixCol, int _matrixRow, Matrix _parent) {

    myID = _id;
    nProcCol = _nProcCol;
    nProcRow = _nProcRow;
    matrixCol = _matrixCol;
    matrixRow = _matrixRow;
    parent = _parent;

    infoSetting();

    setupGUI();
    //updateData();
  }

  public PartitionPanel(int _id, int _nProcCol, int _nProcRow, int _matrixCol, int _matrixRow, Color _colorBackground, Color _colorBoundary, Color _colorBuffer, Color _colorLocalOld, Color _colorLocalNew, Color _colorLocalNewChanged, Matrix _parent, AnimationPanel _animator) {

    myID = _id;
    nProcCol = _nProcCol;
    nProcRow = _nProcRow;
    matrixCol = _matrixCol;
    matrixRow = _matrixRow;
    parent = _parent;
    animator = _animator;

    colorBackground = _colorBackground;
    colorBoundary   = _colorBoundary;
    colorBuffer     = _colorBuffer;
    colorLocalOld   = _colorLocalOld;
    colorLocalNew   = _colorLocalNew;
    colorLocalNewChanged = _colorLocalNewChanged;

    infoSetting();

    setupGUI();
    //updateData();
    //updateMatrix();
    initMatrix();
  }

  protected void infoSetting() {
    localCol = matrixCol / nProcCol;
    localRow = matrixRow / nProcRow;

    procX = (int)(myID % nProcCol);
    procY = (int)(myID / nProcCol);

    startRowIndex = 0;
    startColIndex = 0;

    endRowIndex = localRow;
    endColIndex = localCol;

    // decide the local data and neighbors
    if ( procY == 0 ) {
      northNeighborID = -1;
      northNeighbor = false;
      startRowIndex++;
    }
    else {
      northNeighborID =  procX + (procY-1) * nProcCol;
      northNeighbor = true;
    }

    if ( procY == nProcRow-1 ) {
      southNeighborID = -1;
      southNeighbor = false;
      endRowIndex--;
    }
    else {
      southNeighborID = procX + (procY+1) * nProcCol;
      southNeighbor = true;
    }

    if ( procX == nProcCol-1 ) {
      eastNeighborID = -1;
      eastNeighbor = false;
      endColIndex--;
    }
    else {
      eastNeighborID = procX + 1 + (procY * nProcCol);
      eastNeighbor = true;
    }

    if ( procX == 0 ) {
      westNeighborID = -1;
      westNeighbor = false;
      startColIndex++;
    }
    else {
      westNeighborID = procX - 1 + (procY * nProcCol);
      westNeighbor = true;
    }

    // gettinig the matrix data
    double originalData[][];
    originalData = new double[matrixCol][matrixRow];
    originalData = parent.getMatrix();
    matrixData = new double[localCol+2][localRow+2];
    newMatrixData = new double[localCol+2][localRow+2];

    for ( int i = 0 ; i < localRow ; i++ ) {
      int tmpRow = procY * localRow + i;
      for ( int j = 0 ; j < localCol ; j++ ) {
	int tmpCol = procX * localCol + j;
	matrixData[j+1][i+1] = originalData[tmpCol][tmpRow];
	newMatrixData[j+1][i+1] = originalData[tmpCol][tmpRow];
      }
    }

    northBufferData = new double[localCol+1];
    southBufferData = new double[localCol+1];
    eastBufferData  = new double[localRow+1];
    westBufferData  = new double[localRow+1];

    for ( int i = 1 ; i < localRow+1 ; i++ ) {
      String tmpRow = i + ":  ";
      for ( int j = 1 ; j < localCol+1 ; j++ ) {
	tmpRow = tmpRow + matrixData[j][i] + "  ";
      }
    }
  }

  protected void redoGUI() {
    removeAll();
    setupGUI();
  }

  protected void setupGUI() {        

    setLayout(new BorderLayout(5, 5));

    // Local Matrix
    OutlinePanel outBoxCenter = new OutlinePanel(1);
    processor = new ProcessorPanel(localRow, localCol, colorLocalOld, colorLocalNew, colorLocalNewChanged, colorBoundary, colorBackground, startRowIndex, endRowIndex, startColIndex, endColIndex, northNeighbor, southNeighbor, eastNeighbor, westNeighbor, this);
    add("Center", processor);

    // North Buffer
    northBuffer = new RowBuffer(localCol, startColIndex, endColIndex, northNeighbor, colorBackground, colorBuffer);
    add("North", northBuffer);

    // South Buffer
    southBuffer = new RowBuffer(localCol, startColIndex, endColIndex, southNeighbor, colorBackground, colorBuffer);
    add("South", southBuffer);

    // East Buffer
    eastBuffer = new ColumnBuffer(localRow, startRowIndex, endRowIndex, eastNeighbor, colorBackground, colorBuffer);
    add("East", eastBuffer);

    // West Buffer
    westBuffer = new ColumnBuffer(localRow, startRowIndex, endRowIndex, westNeighbor, colorBackground, colorBuffer);
    add("West", westBuffer);
  }

  public void updateData() {
    updateBuffer();
    updateMatrix();
  }

  public void initMatrix() {
    initBuffer();
    updateMatrix();
  }

  public void initBuffer() {
    Vector tmpBuffer =  new Vector();

    // update north buffer
    for ( int i = 1 ; i < localCol+1 ; i++ ) {
      String tmpS = null;
      if ( northNeighbor ) {
	tmpS = "N/A";
      }
      else {
	Double tmpD = new Double(matrixData[i][0]);
	tmpS = tmpD.toString();
      }
      tmpBuffer.addElement(tmpS);
    }
    northBuffer.setText(tmpBuffer);
    tmpBuffer.removeAllElements();

    // update south buffer
    for ( int i = 1 ; i < localCol+1 ; i++ ) {
      String tmpS = null;
      if ( southNeighbor ) {
	tmpS = "N/A";
      }
      else {
	Double tmpD = new Double(matrixData[i][localRow+1]);
	tmpS = tmpD.toString();
      }
      tmpBuffer.addElement(tmpS);
    }
    southBuffer.setText(tmpBuffer);
    tmpBuffer.removeAllElements();

    // update east buffer
    for ( int i = 1 ; i < localRow+1 ; i++ ) {
      String tmpS = null;
      if ( eastNeighbor ) {
	tmpS = "N/A";
      }
      else {
	Double tmpD = new Double(matrixData[localCol+1][i]);
	tmpS = tmpD.toString();
      }
      tmpBuffer.addElement(tmpS);
    }
    eastBuffer.setText(tmpBuffer);
    tmpBuffer.removeAllElements();

    // update west buffer
    for ( int i = 1 ; i < localRow+1 ; i++ ) {
      String tmpS = null;
      if ( westNeighbor ) {
	tmpS = "N/A";
      }
      else {
	Double tmpD = new Double(matrixData[0][i]);
	tmpS = tmpD.toString();
      }
      tmpBuffer.addElement(tmpS);
    }
    westBuffer.setText(tmpBuffer);
    tmpBuffer.removeAllElements();
  }

  public void updateBuffer() {
    Vector tmpBuffer =  new Vector();

    // update north buffer
    for ( int i = 1 ; i < localCol+1 ; i++ ) {
      Double tmpD = new Double(matrixData[i][0]);
      String tmpS = tmpD.toString();
      tmpBuffer.addElement(tmpS);
    }
    northBuffer.setText(tmpBuffer);
    tmpBuffer.removeAllElements();

    // update south buffer
    for ( int i = 1 ; i < localCol+1 ; i++ ) {
      Double tmpD = new Double(matrixData[i][localRow+1]);
      String tmpS = tmpD.toString();
      tmpBuffer.addElement(tmpS);
    }
    southBuffer.setText(tmpBuffer);
    tmpBuffer.removeAllElements();

    // update east buffer
    for ( int i = 1 ; i < localRow+1 ; i++ ) {
      Double tmpD = new Double(matrixData[localCol+1][i]);
      String tmpS = tmpD.toString();
      tmpBuffer.addElement(tmpS);
    }
    eastBuffer.setText(tmpBuffer);
    tmpBuffer.removeAllElements();

    // update west buffer
    for ( int i = 1 ; i < localRow+1 ; i++ ) {
      Double tmpD = new Double(matrixData[0][i]);
      String tmpS = tmpD.toString();
      tmpBuffer.addElement(tmpS);
    }
    westBuffer.setText(tmpBuffer);
    tmpBuffer.removeAllElements();

    //processor.initialBuffer();
  }

  public void returnBufferColor() {
    northBuffer.returnColor();
    southBuffer.returnColor();
    eastBuffer.returnColor();
    westBuffer.returnColor();
  }



  public void updateMatrix() {
    processor.setText();
  }

  public void startProcessor() {
    //updateData();
    updateBuffer();
    processor.processorThread.start();
  }

  public void suspendProcessor() {
    processor.demoSuspend();
  }

  public void resumeProcessor() {
    processor.demoResume();
  }

  public void nextStepProcessor() {
    processor.stepResume();
  }

  public void setDemoSpeed(int speed) {
    processor.demoSpeed = speed * 10;
  }

  public void sendData() {
    //parent.printLine("I'm sending " + myID);
    // send to north neighbor
    if ( northNeighbor ) {
      Vector data2North = new Vector();
      for ( int j = 1 ; j < localCol+1 ; j++ ) {
	Double tmpD = new Double(matrixData[j][1]);
	String tmpS = tmpD.toString();
	data2North.addElement(tmpS);
      }
      //((PartitionPanel)parent.processors.elementAt(northNeighborID)).sendToBuffer(myID, data2North);
      AnimationPanel tmpAnimation = (AnimationPanel)parent.processors.elementAt(northNeighborID);
      ((PartitionPanel)tmpAnimation.processMain).sendToBuffer(myID, data2North);
      animator.northAnimate.redraw(true);
      //parent.printLine(myID + "send data " + data2North);
    }

    // send to south neighbor
    if ( southNeighbor ) {
      Vector data2South = new Vector();
      for ( int j = 1 ; j < localCol+1 ; j++ ) {
	Double tmpD = new Double(matrixData[j][localRow]);
	String tmpS = tmpD.toString();
	data2South.addElement(tmpS);
      }
      //((PartitionPanel)parent.processors.elementAt(southNeighborID)).sendToBuffer(myID, data2South);
      AnimationPanel tmpAnimation = (AnimationPanel)parent.processors.elementAt(southNeighborID);
      ((PartitionPanel)tmpAnimation.processMain).sendToBuffer(myID, data2South);
      animator.southAnimate.redraw(true);
      //parent.printLine(myID + "s end data " + data2South);
    }

    // send to east neighbor
    if ( eastNeighbor ) {
      Vector data2East = new Vector();
      for ( int i = 1 ; i < localRow+1 ; i++ ) {
	Double tmpD = new Double(matrixData[localCol][i]);
	String tmpS = tmpD.toString();
	data2East.addElement(tmpS);
      }
      //((PartitionPanel)parent.processors.elementAt(eastNeighborID)).sendToBuffer(myID, data2East);
      AnimationPanel tmpAnimation = (AnimationPanel)parent.processors.elementAt(eastNeighborID);
      ((PartitionPanel)tmpAnimation.processMain).sendToBuffer(myID, data2East);
      animator.eastAnimate.redraw(true);
      //parent.printLine(myID + "s end data " + data2East);
    }

    // send to west neighbor
    if ( westNeighbor ) {
      Vector data2West = new Vector();
      for ( int i = 1 ; i < localRow+1 ; i++ ) {
	Double tmpD = new Double(matrixData[1][i]);
	String tmpS = tmpD.toString();
	data2West.addElement(tmpS);
      }
      //((PartitionPanel)parent.processors.elementAt(westNeighborID)).sendToBuffer(myID, data2West);
      AnimationPanel tmpAnimation = (AnimationPanel)parent.processors.elementAt(westNeighborID);
      ((PartitionPanel)tmpAnimation.processMain).sendToBuffer(myID, data2West);
      animator.westAnimate.redraw(true);
      //parent.printLine(myID + "s end data " + data2West);
    }
  }

  public void sendToBuffer(int sourceID, Vector receiveData) {
    //parent.printLine(myID + "  receive data from " + sourceID + "  " + receiveData);
    if (sourceID == southNeighborID) {
      for ( int j = 1 ; j < localCol+1 ; j++ ) {
	String tmpS = (String)receiveData.elementAt(j-1);
	southBufferData[j] = Double.valueOf(tmpS).doubleValue();
      }
      processor.southBufferFilled = true;
    }
    else if (sourceID == northNeighborID) {
      for ( int j = 1 ; j < localCol+1 ; j++ ) {
	String tmpS = (String)receiveData.elementAt(j-1);
	northBufferData[j] = Double.valueOf(tmpS).doubleValue();
      }
      processor.northBufferFilled = true;
    }
    else if (sourceID == eastNeighborID) {
      for ( int i = 1 ; i < localRow+1 ; i++ ) {
	String tmpS = (String)receiveData.elementAt(i-1);
	eastBufferData[i] = Double.valueOf(tmpS).doubleValue();
      }
      processor.eastBufferFilled = true;
    }
    else if (sourceID == westNeighborID) {
      for ( int i = 1 ; i < localRow+1 ; i++ ) {
	String tmpS = (String)receiveData.elementAt(i-1);
	westBufferData[i] = Double.valueOf(tmpS).doubleValue();
      }
      processor.westBufferFilled = true;
    }
    if ( processor.northBufferFilled &&
	 processor.southBufferFilled &&
	 processor.eastBufferFilled &&
	 processor.westBufferFilled ) {
      processor.allBufferFilled();
    }
  }

  public void receiveData() {
    Vector tmpBuffer = new Vector();
    String tmpS = null;

    if ( southNeighbor ) {
      tmpBuffer.removeAllElements();
      for ( int j = 1 ; j < localCol+1 ; j++ ) {
	matrixData[j][localRow+1] = southBufferData[j];
	Double tmpD = new Double(matrixData[j][localRow+1]);
	tmpS = tmpD.toString();
	tmpBuffer.addElement(tmpS);
      }
      southBuffer.setText(tmpBuffer);
      southBuffer.updateColor();
      AnimationPanel tmpAnimation = (AnimationPanel)parent.processors.elementAt(southNeighborID);
      ((AnimationCanvas)tmpAnimation.northAnimate).redraw(false);
    }
    if ( northNeighbor ) {
      tmpBuffer.removeAllElements();
      for ( int j = 1 ; j < localCol+1 ; j++ ) {
	matrixData[j][0] = northBufferData[j];
	Double tmpD = new Double(matrixData[j][0]);
	tmpS = tmpD.toString();
	tmpBuffer.addElement(tmpS);
      }
      northBuffer.setText(tmpBuffer);
      northBuffer.updateColor();
      AnimationPanel tmpAnimation = (AnimationPanel)parent.processors.elementAt(northNeighborID);
      ((AnimationCanvas)tmpAnimation.southAnimate).redraw(false);
    }
    if ( eastNeighbor ) {
      tmpBuffer.removeAllElements();
      for ( int i = 1 ; i < localRow+1 ; i++ ) {
	matrixData[localCol+1][i] = eastBufferData[i];
	Double tmpD = new Double(matrixData[localCol+1][i]);
	tmpS = tmpD.toString();
	tmpBuffer.addElement(tmpS);
      }
      eastBuffer.setText(tmpBuffer);
      eastBuffer.updateColor();
      AnimationPanel tmpAnimation = (AnimationPanel)parent.processors.elementAt(eastNeighborID);
      ((AnimationCanvas)tmpAnimation.westAnimate).redraw(false);
    }
    if ( westNeighbor ) {
      tmpBuffer.removeAllElements();
      for ( int i = 1 ; i < localRow+1 ; i++ ) {
	matrixData[0][i] = westBufferData[i];
	Double tmpD = new Double(matrixData[0][i]);
	tmpS = tmpD.toString();
	tmpBuffer.addElement(tmpS);
      }
      westBuffer.setText(tmpBuffer);
      westBuffer.updateColor();
      AnimationPanel tmpAnimation = (AnimationPanel)parent.processors.elementAt(westNeighborID);
      ((AnimationCanvas)tmpAnimation.eastAnimate).redraw(false);
    }
  }

  protected int myID = 0;
  protected int nProcCol = 0;
  protected int nProcRow = 0;
  protected int matrixCol = 0;
  protected int matrixRow = 0;
  protected Matrix parent = null;
  protected AnimationPanel animator = null;

  protected int localCol = 0;
  protected int localRow = 0;

  protected int procX = 0;
  protected int procY = 0;

  protected boolean northNeighbor = true;
  protected boolean southNeighbor = true;
  protected boolean eastNeighbor = true;
  protected boolean westNeighbor = true;

  protected Color colorBackground;
  protected Color colorBoundary;
  protected Color colorBuffer;
  protected Color colorLocalOld;
  protected Color colorLocalNew;
  protected Color colorLocalNewChanged;

  protected int northNeighborID = -1;
  protected int southNeighborID = -1;
  protected int eastNeighborID = -1;
  protected int westNeighborID = -1;

  protected ProcessorPanel processor;
  protected RowBuffer northBuffer;
  protected RowBuffer southBuffer;
  protected ColumnBuffer eastBuffer;
  protected ColumnBuffer westBuffer;

  protected double matrixData[][];
  protected double newMatrixData[][];

  protected int startRowIndex = 0;
  protected int startColIndex = 0;

  protected int endRowIndex = 0;
  protected int endColIndex = 0;

  protected double northBufferData[];
  protected double southBufferData[];
  protected double eastBufferData[];
  protected double westBufferData[];
}
